Type: Package
Package: systemfonts
Title: System Native Font Finding
Version: 1.3.2
Authors@R: c(
    person("Thomas Lin", "Pedersen", , "thomas.pedersen@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5147-4711")),
    person("Jeroen", "Ooms", , "jeroen@berkeley.edu", role = "aut",
           comment = c(ORCID = "0000-0002-4035-0289")),
    person("Devon", "Govett", role = "aut",
           comment = "Author of font-manager"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Provides system native access to the font catalogue. As font
    handling varies between systems it is difficult to correctly locate
    installed fonts across different operating systems. The 'systemfonts'
    package provides bindings to the native libraries on Windows, macOS
    and Linux for finding font files that can then be used further by e.g.
    graphic devices. The main use is intended to be from compiled code but
    'systemfonts' also provides access from R.
License: MIT + file LICENSE
URL: https://github.com/r-lib/systemfonts,
        https://systemfonts.r-lib.org
BugReports: https://github.com/r-lib/systemfonts/issues
Depends: R (>= 3.2.0)
Imports: base64enc, grid, jsonlite, lifecycle, tools, utils
Suggests: covr, farver, ggplot2, graphics, knitr, ragg, rmarkdown,
        svglite, testthat (>= 2.1.0)
LinkingTo: cpp11 (>= 0.2.1)
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/Needs/website: tidyverse/tidytemplate
Config/usethis/last-upkeep: 2025-04-23
Encoding: UTF-8
RoxygenNote: 7.3.2
SystemRequirements: fontconfig, freetype2
NeedsCompilation: yes
Packaged: 2026-03-05 12:24:52 UTC; thomas
Author: Thomas Lin Pedersen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>),
  Jeroen Ooms [aut] (ORCID: <https://orcid.org/0000-0002-4035-0289>),
  Devon Govett [aut] (Author of font-manager),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Thomas Lin Pedersen <thomas.pedersen@posit.co>
Repository: CRAN
Date/Publication: 2026-03-05 13:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 05:41:30 UTC; unix
Archs: systemfonts.so.dSYM
