# Generated by roxygen2: do not edit by hand

S3method("$",font_variation)
S3method("$<-",font_variation)
S3method("[",font_variation)
S3method("[<-",font_variation)
S3method("[[",font_variation)
S3method("[[<-",font_variation)
S3method(c,font_feature)
S3method(format,font_feature)
S3method(format,font_variation)
S3method(length,font_feature)
S3method(length,font_variation)
S3method(names,font_variation)
S3method(print,font_feature)
S3method(print,font_variation)
export(add_fonts)
export(as_font_weight)
export(as_font_width)
export(clear_local_fonts)
export(clear_registry)
export(font_fallback)
export(font_feature)
export(font_info)
export(font_variation)
export(fonts_as_import)
export(get_fallback)
export(get_from_font_squirrel)
export(get_from_google_fonts)
export(glyph_info)
export(glyph_outline)
export(glyph_raster)
export(glyph_raster_grob)
export(match_font)
export(match_fonts)
export(plot_glyph_stats)
export(register_font)
export(register_variant)
export(registry_fonts)
export(require_font)
export(reset_font_cache)
export(scan_local_fonts)
export(search_web_fonts)
export(shape_string)
export(str_split_emoji)
export(string_metrics_dev)
export(string_width)
export(string_widths_dev)
export(system_fonts)
importFrom(lifecycle,deprecated)
useDynLib(systemfonts, .registration = TRUE)
