## -----------------------------------------------------------------------------
knitr::opts_chunk$set(
  dev = "ragg_png",
  dpi = 144,
  collapse = TRUE,
  comment = "#>",
  fig.asp = NULL,
  fig.height = 4.326,
  fig.width = 7,
  eval = FALSE
)

## -----------------------------------------------------------------------------
# st <- marquee::classic_style(30, body_font = "Avenir Next", lineheight = 1, margin = marquee::trbl(4)) |>
#   marquee::modify_style("anul", weight = "thin") |>
#   marquee::modify_style("anub", weight = "ultrabold")
# text <- paste(
#   "{.anul Avenir Next Ultra Light}  ",
#   "{.anul *Avenir Next Ultra Light Italic*}  ",
#   "Avenir Next  ",
#   "*Avenir Next Italic*  ",
#   "{.anub Avenir Next Ultra Bold}  ",
#   "{.anub *Avenir Next Ultra Bold Italic*}  ",
#   sep = "\n"
# )
# grid::grid.draw(
#   marquee::marquee_grob(text, st)
# )

## -----------------------------------------------------------------------------
# systemfonts::plot_glyph_stats("j", family = "Helvetica", size = 30)

