# Generated by cpp11: do not edit by hand

dev_string_widths_c <- function(string, family, face, size, cex, unit) {
  .Call(`_systemfonts_dev_string_widths_c`, string, family, face, size, cex, unit)
}

dev_string_metrics_c <- function(string, family, face, size, cex, unit) {
  .Call(`_systemfonts_dev_string_metrics_c`, string, family, face, size, cex, unit)
}

load_emoji_codes_c <- function(all, default_text, base_mod) {
  invisible(.Call(`_systemfonts_load_emoji_codes_c`, all, default_text, base_mod))
}

emoji_split_c <- function(string, path, index) {
  .Call(`_systemfonts_emoji_split_c`, string, path, index)
}

get_fallback_c <- function(path, index, string, variations) {
  .Call(`_systemfonts_get_fallback_c`, path, index, string, variations)
}

add_local_fonts <- function(paths) {
  .Call(`_systemfonts_add_local_fonts`, paths)
}

clear_local_fonts_c <- function() {
  invisible(.Call(`_systemfonts_clear_local_fonts_c`))
}

match_font_c <- function(family, italic, bold) {
  .Call(`_systemfonts_match_font_c`, family, italic, bold)
}

locate_fonts_c <- function(family, italic, weight, width) {
  .Call(`_systemfonts_locate_fonts_c`, family, italic, weight, width)
}

system_fonts_c <- function() {
  .Call(`_systemfonts_system_fonts_c`)
}

reset_font_cache_c <- function() {
  invisible(.Call(`_systemfonts_reset_font_cache_c`))
}

get_font_info_c <- function(path, index, size, res, variations) {
  .Call(`_systemfonts_get_font_info_c`, path, index, size, res, variations)
}

get_glyph_info_c <- function(glyphs, path, index, size, res, variations) {
  .Call(`_systemfonts_get_glyph_info_c`, glyphs, path, index, size, res, variations)
}

get_glyph_outlines <- function(glyph, path, index, size, variations, tolerance, verbose) {
  .Call(`_systemfonts_get_glyph_outlines`, glyph, path, index, size, variations, tolerance, verbose)
}

get_glyph_bitmap <- function(glyph, path, index, size, res, variations, color, verbose) {
  .Call(`_systemfonts_get_glyph_bitmap`, glyph, path, index, size, res, variations, color, verbose)
}

register_font_c <- function(family, paths, indices, features, settings) {
  invisible(.Call(`_systemfonts_register_font_c`, family, paths, indices, features, settings))
}

clear_registry_c <- function() {
  invisible(.Call(`_systemfonts_clear_registry_c`))
}

registry_fonts_c <- function() {
  .Call(`_systemfonts_registry_fonts_c`)
}

axes_to_tags <- function(axes) {
  .Call(`_systemfonts_axes_to_tags`, axes)
}

tags_to_axes <- function(tags) {
  .Call(`_systemfonts_tags_to_axes`, tags)
}

values_to_fixed <- function(values) {
  .Call(`_systemfonts_values_to_fixed`, values)
}

fixed_to_values <- function(fixed) {
  .Call(`_systemfonts_fixed_to_values`, fixed)
}

get_string_shape_c <- function(string, id, path, index, size, res, lineheight, align, hjust, vjust, width, tracking, indent, hanging, space_before, space_after) {
  .Call(`_systemfonts_get_string_shape_c`, string, id, path, index, size, res, lineheight, align, hjust, vjust, width, tracking, indent, hanging, space_before, space_after)
}

get_line_width_c <- function(string, path, index, size, res, include_bearing) {
  .Call(`_systemfonts_get_line_width_c`, string, path, index, size, res, include_bearing)
}
