% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{get_fallback}
\alias{get_fallback}
\title{Get location of the fallback font}
\usage{
get_fallback()
}
\description{
Get location of the fallback font
}
\keyword{internal}
